@@ -17,12 +17,9 @@
 #define SOUP_IS_CONNECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), SOUP_TYPE_CONNECTION))
 #define SOUP_CONNECTION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SOUP_TYPE_CONNECTION, SoupConnectionClass))
 
-typedef struct SoupConnectionPrivate SoupConnectionPrivate;
-
 struct SoupConnection {
 	GObject parent;
 
-	SoupConnectionPrivate *priv;
 };
 
 typedef struct {
@@ -46,16 +43,26 @@ typedef struct {
 GType soup_connection_get_type (void);
 
 
+/**
+ * SoupConnectionCallback:
+ * @conn: the #SoupConnection
+ * @status: an HTTP status code indicating success or failure
+ * @data: the data passed to soup_connection_connect_async()
+ *
+ * The callback function passed to soup_connection_connect_async().
+ **/
+typedef void  (*SoupConnectionCallback)        (SoupConnection   *conn,
+						guint             status,
+						gpointer          data);
+
+
 #define SOUP_CONNECTION_ORIGIN_URI      "origin-uri"
 #define SOUP_CONNECTION_PROXY_URI       "proxy-uri"
 #define SOUP_CONNECTION_SSL_CREDENTIALS "ssl-creds"
+#define SOUP_CONNECTION_MESSAGE_FILTER  "message-filter"
 
 SoupConnection *soup_connection_new            (const char       *propname1,
-						...);
-
-typedef void  (*SoupConnectionCallback)        (SoupConnection   *sock,
-						guint             status,
-						gpointer          data);
+						...) G_GNUC_NULL_TERMINATED;
 
 void            soup_connection_connect_async  (SoupConnection   *conn,
 						SoupConnectionCallback callback,
@@ -70,6 +77,9 @@ time_t          soup_connection_last_used      (SoupConnection   *conn);
 void            soup_connection_send_request   (SoupConnection   *conn,
 						SoupMessage      *req);
 
+void            soup_connection_reserve        (SoupConnection   *conn);
+void            soup_connection_release        (SoupConnection   *conn);
+
 /* protected */
 void            soup_connection_authenticate   (SoupConnection   *conn,
 						SoupMessage      *msg,
