@@ -17,12 +17,9 @@
 #define SOUP_IS_SERVER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), SOUP_TYPE_SERVER))
 #define SOUP_SERVER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SOUP_TYPE_SERVER, SoupServerClass))
 
-typedef struct SoupServerPrivate SoupServerPrivate;
-
 struct SoupServer {
 	GObject parent;
 
-	SoupServerPrivate *priv;
 };
 
 typedef struct {
@@ -69,11 +66,13 @@ struct SoupServerHandler {
 #define SOUP_SERVER_SSL_KEY_FILE  "ssl-key-file"
 
 SoupServer        *soup_server_new            (const char            *optname1,
-					       ...);
+					       ...) G_GNUC_NULL_TERMINATED;
 
 SoupProtocol       soup_server_get_protocol   (SoupServer            *serv);
 guint              soup_server_get_port       (SoupServer            *serv);
 
+SoupSocket        *soup_server_get_listener   (SoupServer            *serv);
+
 void               soup_server_run            (SoupServer            *serv);
 void               soup_server_run_async      (SoupServer            *serv);
 void               soup_server_quit           (SoupServer            *serv);
