@@ -10,9 +10,9 @@
 #include "soup-soap-response.h"
 #include "soup-types.h"
 
-#define PARENT_TYPE G_TYPE_OBJECT
+G_DEFINE_TYPE (SoupSoapResponse, soup_soap_response, G_TYPE_OBJECT)
 
-struct _SoupSoapResponsePrivate {
+typedef struct {
 	/* the XML document */
 	xmlDocPtr xmldoc;
 	xmlNodePtr xml_root;
@@ -20,64 +20,49 @@ struct _SoupSoapResponsePrivate {
 	xmlNodePtr xml_method;
 	xmlNodePtr soap_fault;
 	GList *parameters;
-};
-
-static GObjectClass *parent_class = NULL;
+} SoupSoapResponsePrivate;
+#define SOUP_SOAP_RESPONSE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_SOAP_RESPONSE, SoupSoapResponsePrivate))
 
 static void
 finalize (GObject *object)
 {
-	SoupSoapResponse *response = SOUP_SOAP_RESPONSE (object);
+	SoupSoapResponsePrivate *priv = SOUP_SOAP_RESPONSE_GET_PRIVATE (object);
 
-	/* free memory */
-	if (response->priv->xmldoc) {
-		xmlFreeDoc (response->priv->xmldoc);
-		response->priv->xmldoc = NULL;
-	}
+	if (priv->xmldoc)
+		xmlFreeDoc (priv->xmldoc);
+	if (priv->parameters != NULL)
+		g_list_free (priv->parameters);
 
-	response->priv->xml_root = NULL;
-	response->priv->xml_body = NULL;
-	response->priv->xml_method = NULL;
-
-	if (response->priv->parameters != NULL) {
-		g_list_free (response->priv->parameters);
-		response->priv->parameters = NULL;
-	}
-
-	g_free (response->priv);
-	response->priv = NULL;
-
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (soup_soap_response_parent_class)->finalize (object);
 }
 
 static void
-class_init (SoupSoapResponseClass *klass)
+soup_soap_response_class_init (SoupSoapResponseClass *soup_soap_response_class)
 {
-	GObjectClass *object_class;
+	GObjectClass *object_class = G_OBJECT_CLASS (soup_soap_response_class);
 
-	parent_class = g_type_class_peek_parent (klass);
+	g_type_class_add_private (soup_soap_response_class, sizeof (SoupSoapResponsePrivate));
 
-	object_class = G_OBJECT_CLASS (klass);
 	object_class->finalize = finalize;
 }
 
 static void
-init (SoupSoapResponse *response, SoupSoapResponseClass *klass)
+soup_soap_response_init (SoupSoapResponse *response)
 {
-	response->priv = g_new0 (SoupSoapResponsePrivate, 1);
+	SoupSoapResponsePrivate *priv = SOUP_SOAP_RESPONSE_GET_PRIVATE (response);
 
-	response->priv->xmldoc = xmlNewDoc ("1.0");
+	priv->xmldoc = xmlNewDoc ("1.0");
 }
 
-SOUP_MAKE_TYPE (soup_soap_response, SoupSoapResponse, class_init, init, PARENT_TYPE)
 
 /**
  * soup_soap_response_new:
  *
- * Create a new empty %SoupSoapResponse object, which can be modified with the
- * accessor functions provided with this class.
+ * Create a new empty #SoupSoapResponse object, which can be modified
+ * with the accessor functions provided with this class.
  *
- * Return value: the new %SoupSoapResponse (or %NULL if there was an error).
+ * Return value: the new #SoupSoapResponse (or %NULL if there was an
+ * error).
  */
 SoupSoapResponse *
 soup_soap_response_new (void)
@@ -92,10 +77,11 @@ soup_soap_response_new (void)
  * soup_soap_response_new_from_string:
  * @xmlstr: the XML string to parse.
  *
- * Create a new %SoupSoapResponse object from the XML string contained in
- * @xmlstr.
+ * Create a new #SoupSoapResponse object from the XML string contained
+ * in @xmlstr.
  *
- * Return value: the new %SoupSoapResponse (or %NULL if there was an error).
+ * Return value: the new #SoupSoapResponse (or %NULL if there was an
+ * error).
  */
 SoupSoapResponse *
 soup_soap_response_new_from_string (const char *xmlstr)
@@ -114,61 +100,63 @@ soup_soap_response_new_from_string (const char *xmlstr)
 }
 
 static void
-parse_parameters (SoupSoapResponse *response, xmlNodePtr xml_method)
+parse_parameters (SoupSoapResponsePrivate *priv, xmlNodePtr xml_method)
 {
 	xmlNodePtr tmp;
 
 	for (tmp = xml_method->xmlChildrenNode; tmp != NULL; tmp = tmp->next) {
 		if (!strcmp (tmp->name, "Fault")) {
-			response->priv->soap_fault = tmp;
+			priv->soap_fault = tmp;
 			continue;
 		} else {
 			/* regular parameters */
-			response->priv->parameters = g_list_append (response->priv->parameters, tmp);
+			priv->parameters = g_list_append (priv->parameters, tmp);
 		}
 	}
 }
 
 /**
  * soup_soap_response_from_string:
- * @response: the %SoupSoapResponse object.
+ * @response: the #SoupSoapResponse object.
  * @xmlstr: XML string to parse.
  *
- * Parses the string contained in @xmlstr and sets all properties from it in the
- * @response object.
+ * Parses the string contained in @xmlstr and sets all properties from
+ * it in the @response object.
  *
  * Return value: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
 soup_soap_response_from_string (SoupSoapResponse *response, const char *xmlstr)
 {
+	SoupSoapResponsePrivate *priv;
 	xmlDocPtr old_doc = NULL;
 	xmlNodePtr xml_root, xml_body = NULL, xml_method = NULL;
 
 	g_return_val_if_fail (SOUP_IS_SOAP_RESPONSE (response), FALSE);
+	priv = SOUP_SOAP_RESPONSE_GET_PRIVATE (response);
 	g_return_val_if_fail (xmlstr != NULL, FALSE);
 
 	/* clear the previous contents */
-	if (response->priv->xmldoc)
-		old_doc = response->priv->xmldoc;
+	if (priv->xmldoc)
+		old_doc = priv->xmldoc;
 
 	/* parse the string */
-	response->priv->xmldoc = xmlParseMemory (xmlstr, strlen (xmlstr));
-	if (!response->priv->xmldoc) {
-		response->priv->xmldoc = old_doc;
+	priv->xmldoc = xmlParseMemory (xmlstr, strlen (xmlstr));
+	if (!priv->xmldoc) {
+		priv->xmldoc = old_doc;
 		return FALSE;
 	}
 
-	xml_root = xmlDocGetRootElement (response->priv->xmldoc);
+	xml_root = xmlDocGetRootElement (priv->xmldoc);
 	if (!xml_root) {
-		xmlFreeDoc (response->priv->xmldoc);
-		response->priv->xmldoc = old_doc;
+		xmlFreeDoc (priv->xmldoc);
+		priv->xmldoc = old_doc;
 		return FALSE;
 	}
 
 	if (strcmp (xml_root->name, "Envelope") != 0) {
-		xmlFreeDoc (response->priv->xmldoc);
-		response->priv->xmldoc = old_doc;
+		xmlFreeDoc (priv->xmldoc);
+		priv->xmldoc = old_doc;
 		return FALSE;
 	}
 
@@ -177,8 +165,8 @@ soup_soap_response_from_string (SoupSoapResponse *response, const char *xmlstr)
 		if (strcmp (xml_body->name, "Header") == 0)
 			xml_body = xml_root->xmlChildrenNode->next;
 		if (strcmp (xml_body->name, "Body") != 0) {
-			xmlFreeDoc (response->priv->xmldoc);
-			response->priv->xmldoc = old_doc;
+			xmlFreeDoc (priv->xmldoc);
+			priv->xmldoc = old_doc;
 			return FALSE;
 		}
 
@@ -186,21 +174,21 @@ soup_soap_response_from_string (SoupSoapResponse *response, const char *xmlstr)
 
 		/* read all parameters */
 		if (xml_method)
-			parse_parameters (response, xml_method);
+			parse_parameters (priv, xml_method);
 	}
 
 	xmlFreeDoc (old_doc);
 
-	response->priv->xml_root = xml_root;
-	response->priv->xml_body = xml_body;
-	response->priv->xml_method = xml_method;
+	priv->xml_root = xml_root;
+	priv->xml_body = xml_body;
+	priv->xml_method = xml_method;
 
 	return TRUE;
 }
 
 /**
  * soup_soap_response_get_method_name:
- * @response: the %SoupSoapResponse object.
+ * @response: the #SoupSoapResponse object.
  *
  * Gets the method name from the SOAP response.
  *
@@ -209,31 +197,42 @@ soup_soap_response_from_string (SoupSoapResponse *response, const char *xmlstr)
 const char *
 soup_soap_response_get_method_name (SoupSoapResponse *response)
 {
-	g_return_val_if_fail (SOUP_IS_SOAP_RESPONSE (response), NULL);
-	g_return_val_if_fail (response->priv->xml_method != NULL, NULL);
+	SoupSoapResponsePrivate *priv;
 
-	return (const char *) response->priv->xml_method->name;
+	g_return_val_if_fail (SOUP_IS_SOAP_RESPONSE (response), NULL);
+	priv = SOUP_SOAP_RESPONSE_GET_PRIVATE (response);
+	g_return_val_if_fail (priv->xml_method != NULL, NULL);
+
+	return (const char *) priv->xml_method->name;
 }
 
 /**
  * soup_soap_response_set_method_name:
- * @response: the %SoupSoapResponse object.
+ * @response: the #SoupSoapResponse object.
  * @method_name: the method name to set.
  *
- * Sets the method name on the given %SoupSoapResponse.
+ * Sets the method name on the given #SoupSoapResponse.
  */
 void
 soup_soap_response_set_method_name (SoupSoapResponse *response, const char *method_name)
 {
+	SoupSoapResponsePrivate *priv;
+
 	g_return_if_fail (SOUP_IS_SOAP_RESPONSE (response));
-	g_return_if_fail (response->priv->xml_method != NULL);
+	priv = SOUP_SOAP_RESPONSE_GET_PRIVATE (response);
+	g_return_if_fail (priv->xml_method != NULL);
 	g_return_if_fail (method_name != NULL);
 
-	xmlNodeSetName (response->priv->xml_method, method_name);
+	xmlNodeSetName (priv->xml_method, method_name);
 }
 
 /**
  * soup_soap_parameter_get_name:
+ * @param: the parameter
+ *
+ * Returns the parameter name.
+ *
+ * Return value: the parameter name.
  */
 const char *
 soup_soap_parameter_get_name (SoupSoapParameter *param)
@@ -245,6 +244,11 @@ soup_soap_parameter_get_name (SoupSoapParameter *param)
 
 /**
  * soup_soap_parameter_get_int_value:
+ * @param: the parameter
+ *
+ * Returns the parameter's (integer) value.
+ *
+ * Return value: the parameter value as an integer
  */
 int
 soup_soap_parameter_get_int_value (SoupSoapParameter *param)
@@ -266,6 +270,12 @@ soup_soap_parameter_get_int_value (SoupSoapParameter *param)
 
 /**
  * soup_soap_parameter_get_string_value:
+ * @param: the parameter
+ *
+ * Returns the parameter's value.
+ *
+ * Return value: the parameter value as a string, which must be freed
+ * by the caller.
  */
 char *
 soup_soap_parameter_get_string_value (SoupSoapParameter *param)
@@ -282,13 +292,13 @@ soup_soap_parameter_get_string_value (SoupSoapParameter *param)
 
 /**
  * soup_soap_parameter_get_first_child:
- * @param: A %SoupSoapParameter.
+ * @param: A #SoupSoapParameter.
  *
- * Gets the first child of the given %SoupSoapParameter. This is used
+ * Gets the first child of the given #SoupSoapParameter. This is used
  * for compound data types, which can contain several parameters
  * themselves.
  *
- * Return value: the first child or NULL if there are no children.
+ * Return value: the first child or %NULL if there are no children.
  */
 SoupSoapParameter *
 soup_soap_parameter_get_first_child (SoupSoapParameter *param)
@@ -300,13 +310,13 @@ soup_soap_parameter_get_first_child (SoupSoapParameter *param)
 
 /**
  * soup_soap_parameter_get_first_child_by_name:
- * @param: A %SoupSoapParameter.
+ * @param: A #SoupSoapParameter.
  * @name: The name of the child parameter to look for.
  *
- * Gets the first child of the given %SoupSoapParameter whose name
- * is @name.
+ * Gets the first child of the given #SoupSoapParameter whose name is
+ * @name.
  *
- * Return value: the first child with the given name or NULL if there
+ * Return value: the first child with the given name or %NULL if there
  * are no children.
  */
 SoupSoapParameter *
@@ -329,7 +339,16 @@ soup_soap_parameter_get_first_child_by_name (SoupSoapParameter *param, const cha
 
 /**
  * soup_soap_parameter_get_next_child:
- * @param: A %SoupSoapParameter.
+ * @param: A #SoupSoapParameter.
+ *
+ * Gets the next sibling of the given #SoupSoapParameter. This is used
+ * for compound data types, which can contain several parameters
+ * themselves.
+ *
+ * FIXME: the name of this method is wrong
+ *
+ * Return value: the next sibling, or %NULL if there are no more
+ * siblings.
  */
 SoupSoapParameter *
 soup_soap_parameter_get_next_child (SoupSoapParameter *param)
@@ -341,6 +360,15 @@ soup_soap_parameter_get_next_child (SoupSoapParameter *param)
 
 /**
  * soup_soap_parameter_get_next_child_by_name:
+ * @param: A #SoupSoapParameter.
+ * @name: The name of the sibling parameter to look for.
+ *
+ * Gets the next sibling of the given #SoupSoapParameter whose name is
+ * @name.
+ *
+ * FIXME: the name of this method is wrong
+ *
+ * Return value: the next sibling with the given name, or %NULL
  */
 SoupSoapParameter *
 soup_soap_parameter_get_next_child_by_name (SoupSoapParameter *param,
@@ -363,8 +391,12 @@ soup_soap_parameter_get_next_child_by_name (SoupSoapParameter *param,
 
 /**
  * soup_soap_parameter_get_property:
- * @param:
+ * @param: the parameter
  * @prop_name: Name of the property to retrieve.
+ *
+ * Returns the named property of @param.
+ *
+ * Return value: the property, which must be freed by the caller.
  */
 char *
 soup_soap_parameter_get_property (SoupSoapParameter *param, const char *prop_name)
@@ -383,62 +415,66 @@ soup_soap_parameter_get_property (SoupSoapParameter *param, const char *prop_nam
 
 /**
  * soup_soap_response_get_parameters:
- * @response: the %SoupSoapResponse object.
+ * @response: the #SoupSoapResponse object.
  *
  * Returns the list of parameters received in the SOAP response.
  *
- * Return value: the list of parameters, represented in
- * SoupSoapParameter's, which is an opaque type used to
- * represent a parameter in the SOAP response.
+ * Return value: a list of #SoupSoapParameter
  */
 const GList *
 soup_soap_response_get_parameters (SoupSoapResponse *response)
 {
-	g_return_val_if_fail (SOUP_IS_SOAP_RESPONSE (response), NULL);
+	SoupSoapResponsePrivate *priv;
 
-	return (const GList *) response->priv->parameters;
+	g_return_val_if_fail (SOUP_IS_SOAP_RESPONSE (response), NULL);
+	priv = SOUP_SOAP_RESPONSE_GET_PRIVATE (response);
+
+	return (const GList *) priv->parameters;
 }
 
 /**
  * soup_soap_response_get_first_parameter:
- * @response: the %SoupSoapResponse object.
+ * @response: the #SoupSoapResponse object.
  *
  * Retrieves the first parameter contained in the SOAP response.
  *
- * Return value: a %SoupSoapParameter representing the
- * first parameter. This is an opaque type used to
- * represent a parameter in the SOAP response.
+ * Return value: a #SoupSoapParameter representing the first
+ * parameter, or %NULL if there are no parameters.
  */
 SoupSoapParameter *
 soup_soap_response_get_first_parameter (SoupSoapResponse *response)
 {
-	g_return_val_if_fail (SOUP_IS_SOAP_RESPONSE (response), NULL);
+	SoupSoapResponsePrivate *priv;
 
-	return response->priv->parameters ? response->priv->parameters->data : NULL;
+	g_return_val_if_fail (SOUP_IS_SOAP_RESPONSE (response), NULL);
+	priv = SOUP_SOAP_RESPONSE_GET_PRIVATE (response);
+
+	return priv->parameters ? priv->parameters->data : NULL;
 }
 
 /**
  * soup_soap_response_get_first_parameter_by_name:
- * @response: the %SoupSoapResponse object.
+ * @response: the #SoupSoapResponse object.
  * @name: the name of the parameter to look for.
  *
  * Retrieves the first parameter contained in the SOAP response whose
  * name is @name.
  *
- * Return value: a %SoupSoapParameter representing the
- * first parameter. This is an opaque type used to
- * represent a parameter in the SOAP response.
+ * Return value: a #SoupSoapParameter representing the first parameter
+ * with the given name, or %NULL.
  */
 SoupSoapParameter *
 soup_soap_response_get_first_parameter_by_name (SoupSoapResponse *response,
 						const char *name)
 {
+	SoupSoapResponsePrivate *priv;
 	GList *l;
 
 	g_return_val_if_fail (SOUP_IS_SOAP_RESPONSE (response), NULL);
+	priv = SOUP_SOAP_RESPONSE_GET_PRIVATE (response);
 	g_return_val_if_fail (name != NULL, NULL);
 
-	for (l = response->priv->parameters; l != NULL; l = l->next) {
+	for (l = priv->parameters; l != NULL; l = l->next) {
 		SoupSoapParameter *param = (SoupSoapParameter *) l->data;
 
 		if (!strcmp (name, param->name))
@@ -450,23 +486,26 @@ soup_soap_response_get_first_parameter_by_name (SoupSoapResponse *response,
 
 /**
  * soup_soap_response_get_next_parameter:
- * @response: the %SoupSoapResponse object.
+ * @response: the #SoupSoapResponse object.
  * @from: the parameter to start from.
  *
- * Retrieves the parameter following @from in the %SoupSoapResponse object.
+ * Retrieves the parameter following @from in the #SoupSoapResponse
+ * object.
  *
- * Return value: a %SoupSoapParameter representing the parameter.
+ * Return value: a #SoupSoapParameter representing the parameter.
  */
 SoupSoapParameter *
 soup_soap_response_get_next_parameter (SoupSoapResponse *response,
 				       SoupSoapParameter *from)
 {
+	SoupSoapResponsePrivate *priv;
 	GList *l;
 
 	g_return_val_if_fail (SOUP_IS_SOAP_RESPONSE (response), NULL);
+	priv = SOUP_SOAP_RESPONSE_GET_PRIVATE (response);
 	g_return_val_if_fail (from != NULL, NULL);
 
-	l = g_list_find (response->priv->parameters, (gconstpointer) from);
+	l = g_list_find (priv->parameters, (gconstpointer) from);
 	if (!l)
 		return NULL;
 
@@ -475,14 +514,14 @@ soup_soap_response_get_next_parameter (SoupSoapResponse *response,
 
 /**
  * soup_soap_response_get_next_parameter_by_name:
- * @response: the %SoupSoapResponse object.
+ * @response: the #SoupSoapResponse object.
  * @from: the parameter to start from.
  * @name: the name of the parameter to look for.
  *
- * Retrieves the parameter following @from in the %SoupSoapResponse object
- * whose name matches @name.
+ * Retrieves the first parameter following @from in the
+ * #SoupSoapResponse object whose name matches @name.
  *
- * Return value: a %SoupSoapParameter representing the parameter.
+ * Return value: a #SoupSoapParameter representing the parameter.
  */
 SoupSoapParameter *
 soup_soap_response_get_next_parameter_by_name (SoupSoapResponse *response,
